shader{

	code= "
	#define			SCATTERING_INTENSITY 0.5
	uniform vec3	lightPosition;
	uniform vec4	lightColor;
	uniform float	lightRadius;

	void main()
	{
		//This shader implements the volumetric lighting technique proposed in Crytek's presentation
		//'Real-time Atmospheric Effects in Games Revisited' with the necessary mathematical derivations
		//provided by Miles Macklin from his blog: http://mmack.wordpress.com/2010/05/29/in-scattering-demo/.
		//A few additions were made to the original technique to obtain a more natural effect, limit the
		//light radius and avoid artifacts.

		gl_TexCoord[0].st = gl_TexCoord[0].st / gl_TexCoord[0].w * 0.5 + 0.5;

		vec3 position = reconstructPositionFromDepth(gl_TexCoord[0].st);
		vec3 viewVector = normalize(position - cameraPosition);
		float d = length(position - cameraPosition);

		//Calculate the amount of light scattered towards the viewer. Check Miles Macklin blog
		//for the explanation on the mathematical derivation of this effect.
		vec3 cameraToLightVector = cameraPosition - lightPosition;
		float b = dot(viewVector, cameraToLightVector);
		float c = dot(cameraToLightVector, cameraToLightVector);
		float aux = sqrt(c - b * b);
		float s = 1.0 / max(aux, 0.001);							//Note: aux is clamped to avoid divisions by zero.
		float scattering = s * (atan((d + b) * s) - atan(b * s));


		//The original effect makes the light spot too bright so we apply an exponential
		//tone mapping operator and an intensity factor to avoid excessive color saturation.
		scattering = (1 - exp(-scattering)) * SCATTERING_INTENSITY;

		
		//Apply light attenuation to the effect so we can limit its
		//range and avoid drawing a fullscreen pass for each light.
		vec3 nearestPoint = cameraPosition + viewVector * max(dot(-cameraToLightVector, viewVector), 0.0);
		float distanceToLight = length(nearestPoint - lightPosition);
		float attenuation = max(1.0 - distanceToLight / lightRadius, 0.0);

		gl_FragColor = lightColor * scattering * attenuation;
	}
	"
	
	includes
	{
		file= resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
	}

	semantics
	{
		lightPosition = LIGHT_POSITION
		lightColor = LIGHT_COLOR
		lightRadius = LIGHT_RADIUS
	}
}