shader{
	
	code= "
	#define PI				3.141592
	#define MAX_RAYS		200
	
	//Spherical Harmonics coefficients.
	#define C1				0.282095
	#define C2				0.488603
	#define C3				1.092548
	#define C4				0.315392
	#define C5				0.546274

	uniform vec3			rays[MAX_RAYS];
	uniform int				numRays; 

	
	varying vec3 position;
	
	void main()
	{
		vec3 L00, L1m1, L10, L11, L2m2, L2m1, L20, L21, L22;
		
		for(int i = 0; i < numRays; ++i)
		{
			vec3 direction = rays[i];			

			if(isSkyAccessibleToRay(position, direction))
			{
				//Get the sky color.
				vec3 skyColor = sampleSkyMap(direction);

				//Get the fog color and density.
				vec4 fogColorAndDensity = getFog(position + direction * 1000.0, position);

				//Blend the sky color and the fog color based on the fog density.
				vec3 color = mix(skyColor, fogColorAndDensity.rgb, fogColorAndDensity.a);
					
				L00 += color;
				L1m1 += color * direction.y;
				L10 += color * direction.z;
				L11 += color * direction.x;
				L2m2 += color * (direction.x*direction.y);
				L2m1 += color * (direction.y*direction.z);
				L20 += color * (3.0*direction.z*direction.z - 1.0);
				L21 += color *  (direction.x*direction.z);
				L22 += color * (direction.x*direction.x - direction.y*direction.y);
			}
		}
		
		//Normalize and weight.
		float weight= 2.0 * PI/numRays;
		L00 *= weight * C1;
		L1m1 *= weight * C2;
		L10 *= weight * C2;
		L11 *= weight * C2;
		L2m2 *= weight * C3;
		L2m1 *= weight * C3;
		L20 *= weight * C4;
		L21 *= weight * C3;
		L22 *= weight * C5;
		
		gl_FragData[0] = vec4(L00.x, L00.y, L00.z, L1m1.x);
		gl_FragData[1] = vec4(L1m1.y, L1m1.z, L10.x, L10.y);
		gl_FragData[2] = vec4(L10.z, L11.x, L11.y, L11.z);
		gl_FragData[3] = vec4(L2m2.x, L2m2.y, L2m2.z, L2m1.x);
		gl_FragData[4] = vec4(L2m1.y, L2m1.z, L20.x, L20.y);
		gl_FragData[5] = vec4(L20.z, L21.x, L21.y, L21.z);
		gl_FragData[6] = vec4(L22.x, L22.y, L22.z, 0.0);
	}
	"
	
	includes
	{
		file = resources/shaders/engine/atmosphere/skyLighting/skyMapRead.sh
		file = resources/shaders/engine/atmosphere/skyLighting/skyAccessibility.sh
		file = resources/shaders/engine/atmosphere/fog/fog.sh
	}
	
	semantics
	{
		rays = SKY_LIGHTING_SAMPLING_VECTORS
		numRays = SKY_LIGHTING_NUM_SAMPLING_VECTORS
	}
}