shader{

	code= "
		#define PI					3.141592
		#define MAX_MIPMAP_LEVEL	8			//Max mipmap level for a 256x256 sky map.


		uniform sampler2D skyMapTexture;

		vec3 sampleSkyMap(vec3 direction, float coneAngle = 0.0)
		{		
			//Convert the direction to parapoloid space.
			direction/= length(direction);
			direction.y+= 1.0;
			direction.x/= direction.y;
			direction.z/= direction.y;

			//Compensate for the border that was added to the sky map to
			//allow the smooth transition to black bellow the horizon.
			direction.xz /= 1.2;

			//Calculate the texture coordinates.
			vec2 texCoord = direction.xz * 0.5+ 0.5;

			//The sky dome can be assumed to be a cone with a 180 angle, so we'll
			//calculate how wide is our cone in comparison to the dome.
			float relativeConeAngle = min(coneAngle / PI, 1.0);

			//Convert the relative cone angle to a LOD level. In this step we assume that sampling from
			//level 0 of the texture (the one with 256x256 resolution) corresponds to a 0 wide cone
			//while the last level (the one with 1x1 resolution) corresponds to a 180 wide cone which
			//essentially samples from the entire sky dome.
			float lodLevel = relativeConeAngle * MAX_MIPMAP_LEVEL;
			
			vec3 color = textureLod(skyMapTexture, texCoord, lodLevel).rgb;
			
			return color;
		}
	"
	
	semantics{
		skyMapTexture= SKY_MAP_TEXTURE
	}
}