shader{

	code= "
	uniform vec3			sunLightVector;
	
	void main(){
		
		vec3 normal;
		
		//Convert the texture coordinates from the [0, 1] to the [-1, 1] range.
		normal.xz= (gl_TexCoord[0].xy*2.0- 1.0);

		//We need to leave some space between the sky information and the texture borders
		//so we can do a smooth transition to black bellow the horizon. This transition is
		//mostly important to generate good mipmaps so we can sample the map with cones.
		//The constant used in the scaling has no particular meaning, it was just choosen
		//by trial and error to give a fairly tight fit of enough space for the transition.
		normal.xz *= 1.2;

		//Calculate the Y component.
		normal.y= 0.5- 0.5*(normal.x*normal.x + normal.z*normal.z);
		normal= normalize(normal);		
		
		//Get the atmospheric scattering. Note that the sampling coordinates are clamped so that when
		//sampling bellow the horizon we obtain the scattering as if we were at the horizon level.
		vec3 samplingDirection = normal;
		samplingDirection.y = max(samplingDirection.y, 0.0);
		samplingDirection = normalize(samplingDirection);
		vec3 skyColor= getSkyAtmosphericScattering(samplingDirection, vec3(0.0, 0.0, 0.0), sunLightVector);

		//Perform the smooth transition to black bellow the horizon.
		float bellowHorizon = max(-normal.y, 0.0); 
		float transitionAmount = max(1.0 - bellowHorizon * 5.0, 0.0);
		skyColor *= transitionAmount;
	
		gl_FragColor= vec4(skyColor, 1.0);
	}
	"
	includes{
		file= resources/shaders/engine/atmosphere/skyLighting/atmosphericScattering.sh
	}
	
	
	semantics{
		sunLightVector= SUN_LIGHT_DIRECTION
	}
}