shader{

	code= "
	uniform mat4		decalProjectionMatrix;
	uniform sampler2D	decalColorTexture;
	
	void main(){
		
		//Calculate the screen space position of the pixel.
		gl_TexCoord[0].st /= gl_TexCoord[0].w;
		gl_TexCoord[0].st = gl_TexCoord[0].st * 0.5 + 0.5;

		//Obtain the world space position of the pixel.
		vec3 position = reconstructPositionFromDepth(gl_TexCoord[0].st);

		vec3 decalTexCoord = (decalProjectionMatrix * vec4(position, 1.0)).xyz;
		decalTexCoord = decalTexCoord * 0.5 + 0.5;
		
		//Discard the pixel it it falls outside the projection matrix.
		if(decalTexCoord.x < 0.0 || decalTexCoord.x > 1 || decalTexCoord.y < 0.0 || decalTexCoord.y > 1 || decalTexCoord.z < 0 || decalTexCoord.z > 1)
			discard;

		//Sample the color, transparency and glossiness from textures.
		vec4 colorAndTransparency = texture2D(decalColorTexture, decalTexCoord.st);
		float glossiness = 1.0;

		//Output color and glossiness.
		gl_FragData[0]= vec4(colorAndTransparency.rgb, glossiness);

		//Transparency is outputted to the second draw buffer and used as input to the blending
		//equation using dual-source blending to allow us to keep the alpha of the first buffer as data.
		gl_FragData[1]= vec4(0, 0, 0, colorAndTransparency.a);
	}"

	includes{
		file= resources/shaders/engine/positionFromDepth.sh
	}

	semantics{
		decalProjectionMatrix = DEFERRED_DECAL_PROJECTION_MATRIX
		decalColorTexture = DEFERRED_DECAL_COLOR_TEXTURE
	}
}