shader{

	code= "
	uniform sampler2D	colorTexture;
	uniform sampler2D	normalTexture;
	uniform sampler2D	geometryNormalTexture;
	uniform sampler2D	depthTexture;	
	uniform sampler2D	positionTexture;

	uniform vec3		cameraPosition;
	uniform vec3		frustumUpperLeftVector;
	uniform vec3		frustumUpperRightVector;
	uniform vec3		frustumLowerLeftVector;
	uniform vec3		frustumLowerRightVector;
	
	vec3 getColor(vec2 texCoord)
	{
		return texture(colorTexture, texCoord).rgb;
	}

	vec4 getNormalAndSpecularRoughness(vec2 texCoord)
	{
		return texture(normalTexture, texCoord);
	}

	vec3 getNormal(vec2 texCoord)
	{
		return texture(normalTexture, texCoord).xyz;
	}

	float getSpecularRoughness(vec2 texCoord)
	{
		return texture(normalTexture, texCoord).a;
	}

	vec3 getGeometryNormal(vec2 texCoord)
	{
		return texture(geometryNormalTexture, texCoord).rgb;
	}

	float getSpecularMultiplier(vec2 texCoord)
	{
		return texture(geometryNormalTexture, texCoord).a;
	}

	//Gets the pixel's depth.
	float getPixelDepth(vec2 texcoord)
	{
		return texture(depthTexture, texcoord).x;
	}

	//Get the pixel's position.
	vec3 getPosition(vec2 texcoord)
	{
		return texture(positionTexture, texcoord).xyz;
	}
	
	//Reconstructs the position by fetching the pixel's depth and calculating the interpolated frustum vector.
	vec3 reconstructPositionFromDepth(vec2 texcoord)
	{
		float depth= getPixelDepth(texcoord);

		vec3 aux1 = mix(frustumUpperLeftVector, frustumUpperRightVector, texcoord.x);
		vec3 aux2 = mix(frustumLowerLeftVector, frustumLowerRightVector, texcoord.x);
		vec3 frustumVector = mix(aux2, aux1, texcoord.y);

		return cameraPosition+ frustumVector*depth;		
	}
	
	//Reconstructs the position from the pixel's texture coordinates and the interpolated frustum vector.
	vec3 reconstructPositionFromDepth(vec2 texcoord, vec3 frustumVector)
	{
		float depth= getPixelDepth(texcoord);
		return cameraPosition+ frustumVector*depth;		
	}
	
	//Reconstructs the position from the pixel's depth and the interpolated frustum vector.
	vec3 reconstructPositionFromDepth(float depth, vec3 frustumVector)
	{
		return cameraPosition+ frustumVector*depth;		
	}
	"

	semantics{
		colorTexture= GBUFFER_COLOR_TEXTURE
		normalTexture= GBUFFER_NORMAL_TEXTURE
		geometryNormalTexture= GBUFFER_GEOMETRY_NORMAL_TEXTURE
		depthTexture= GBUFFER_DEPTH_TEXTURE
		positionTexture = GBUFFER_POSITION_TEXTURE
		cameraPosition= CAMERA_POSITION
		frustumUpperLeftVector= POSITION_RECONSTRUCTION_UPPER_LEFT_VECTOR
		frustumUpperRightVector= POSITION_RECONSTRUCTION_UPPER_RIGHT_VECTOR
		frustumLowerLeftVector= POSITION_RECONSTRUCTION_LOWER_LEFT_VECTOR
		frustumLowerRightVector= POSITION_RECONSTRUCTION_LOWER_RIGHT_VECTOR
	}
}