shader{

	code= "
	uniform sampler2D	lightMaskTexture;
	
	void main()
	{
		//Get the material attributes.
		vec3 color = getColor(gl_TexCoord[0].st);
		float specularMultiplier = getSpecularMultiplier(gl_TexCoord[0].st);
		vec4 normalAndSpecularRoughness = getNormalAndSpecularRoughness(gl_TexCoord[0].st);

		//Get the world space position.		
		vec3 position = reconstructPositionFromDepth(gl_TexCoord[0].st, gl_TexCoord[1].xyz);

		//Calculate the view vector.
		vec3 viewVector = normalize(position- cameraPosition);

		//Calculate the sun lighting.
		vec3 lighting = getSunLighting(color, normalAndSpecularRoughness.xyz, specularMultiplier, normalAndSpecularRoughness.w, viewVector);

		//Get the shadow mask.
		float shadow = texture2D(lightMaskTexture, gl_TexCoord[0].st).x;

		gl_FragColor = vec4(lighting * shadow, 1.0);
	}
	"

	includes{
		file= resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
		file= resources/shaders/engine/deferredRendering/sunLighting.sh
	}

	semantics{
		lightMaskTexture= LIGHT_MASK_TEXTURE
	}
}