shader
{
	code= "
	//The deferred sun lighting calculations are used by several effects like
	//the deferred light passes, the deferred raytacing light passes and the
	//light propagation volumes reflective shadow maps. So, to avoid code repetition
	//and to simplify code maintenance, that code is concentrated in this shader.

	uniform vec3		sunLightVector;
	uniform vec4		lightColor;


	vec3 getSunLightingNoSpecular(vec3 color, vec3 normal)
	{
		float diffuse= max(dot(-sunLightVector, normal), 0.0);
		return color * lightColor.rgb * diffuse;
	}


	vec3 getSunLighting(vec3 color, vec3 normal, float specularMultiplier, float specularRoughness, vec3 viewVector)
	{
		vec3 lighting = calculateLighting(color, specularRoughness, specularMultiplier, normal, -sunLightVector, -viewVector, lightColor);
		
		//Calculate the amount of occlusion the fog causes to the sun's light intensity.
		float fogDensity = getFogDensity(-sunLightVector * 1000.0, vec3(0.0));
		float fogOcclusion = (1.0 - fogDensity);
		
		return lighting * fogOcclusion;
	}
	"

	includes
	{
		file= resources/shaders/engine/deferredRendering/lightingModel.sh
		file = resources/shaders/engine/atmosphere/fog/fog.sh
	}

	semantics
	{
		sunLightVector= SUN_LIGHT_DIRECTION
		lightColor= LIGHT_COLOR
	}
}