shader
{
	code= "
	uniform sampler2D	colorTexture;
	uniform sampler2D	normalTexture;

	varying in mat3		tangentSpaceMatrixOut;
	varying in vec3		positionOut;

	void main()
	{
		//Color.
		vec3 color = texture2D(colorTexture, gl_TexCoord[0].st).rgb;

		//Normal.
		vec3 normal = texture2D(normalTexture, gl_TexCoord[0].st).xyz * 2.0 - 1.0;		
		normal = normalize(normal * transpose(tangentSpaceMatrixOut));

		//Calculate the lighting.
		vec3 lighting = getPointLighting(color, positionOut, normal, vec3(0, 0, 0) , 0, 0);

		//Calculate the distance from the light source to the pixel.
		float distance = length(positionOut - lightPosition);

		gl_FragData[0] = vec4(lighting, 1);
		gl_FragData[1] = vec4(normal, 1);
		gl_FragData[2] = distance;
	}"

	includes
	{
		file = "resources/shaders/engine/deferredRendering/pointLighting.sh"
	}

	semantics
	{
		colorTexture = MATERIAL_COLOR_TEXTURE
		normalTexture = MATERIAL_NORMAL_TEXTURE
	}
}