shader{

	code= "

	#define NUM_SAMPLES			4
	#define SAMPLING_DISTANCE	6.0

	uniform sampler2D	colorTexture;
	uniform sampler2D	positionTexture;
	uniform sampler2D	normalTexture;
	uniform vec3		cameraPosition;
	
	void main(){

		float glossiness= texture2D(colorTexture, gl_TexCoord[0].st).a;

		//Calculate the reflections only if the glossiness is greater than zero.
		if(glossiness> 0.0001){

			vec3 position= texture2D(positionTexture, gl_TexCoord[0].st).xyz;
			vec3 normal= texture2D(normalTexture, gl_TexCoord[0].st).xyz;

			vec3 viewVector= normalize(position- cameraPosition);
			vec3 reflectionVector= reflect(viewVector, normal);

			vec3 illumination= 0.0;
			vec3 samplingPosition= position;

			for(int i= 0; i< NUM_SAMPLES; ++i){

				illumination+= sampleLightPropagationVolume(samplingPosition, reflectionVector);
				samplingPosition+= reflectionVector*(SAMPLING_DISTANCE/NUM_SAMPLES);
			}

			illumination/= NUM_SAMPLES;
		
			gl_FragColor= vec4(illumination*glossiness, 0.0);
		}
		else
			gl_FragColor= vec4(0.0, 0.0, 0.0, 0.0);
	}
	"
	
	includes{
		file= resources/shaders/engine/lightPropagationVolumes/propagationVolumeRead.sh
	}
	
	semantics{
		colorTexture= GBUFFER_COLOR_TEXTURE
		positionTexture= GBUFFER_POSITION_TEXTURE
		normalTexture= GBUFFER_NORMAL_TEXTURE
		cameraPosition= CAMERA_POSITION
	}
}