shader{

	code= "
	uniform sampler2D	depthTexture;
	uniform sampler2D	colorTexture;
	uniform sampler2D	positionTexture;
	uniform sampler2D	normalTexture;
	uniform float		propagationVolumeWorldSpaceTexelSize;
	uniform float		propagationVolumeLightsWeight;
	uniform vec3		lightPropagationVolumePosition;
	uniform float		lightPropagationVolumeWidth;
	uniform float		lightPropagationVolumeHeight;
	uniform float		lightPropagationVolumeLength;
	uniform float		propagationVolumeHeightResolution;
	
	varying vec3		lightColor;
	varying vec3		lightDirection;
	
	void main(){

		float linearDepth= texture2D(depthTexture, gl_MultiTexCoord0.st).x;
		vec3 baseColor= texture2D(colorTexture, gl_MultiTexCoord0.st).xyz;
		vec3 position= texture2D(positionTexture, gl_MultiTexCoord0.st).xyz;
		vec3 normal= texture2D(normalTexture, gl_MultiTexCoord0.st).xyz;
		
		//Half cell shifting of the position in the direction of normal to avoid self illumination.
		position+= normal*propagationVolumeWorldSpaceTexelSize*0.5;
		
		vec3 color= baseColor*sampleSkyLightingVolume(position, normal);

		//Multiply the color by the light's weight.
		color*= 0.0005*linearDepth;



		//Injection=======================================================================================


		//Lay down the vertex on the unwrapped texture. This is done by finding to which layer the
		//vertex belongs to based on its Y coordinate and calculating an offset that is applied to
		//the vertex position.
		
		vec3 volumeDimensions= vec3(lightPropagationVolumeWidth, lightPropagationVolumeHeight, lightPropagationVolumeLength);
		vec3 positionInsideVolume= (position- (lightPropagationVolumePosition- volumeDimensions*0.5))/volumeDimensions;

		//Make sure the vertex is inside the layer, if not then
		//it will be ignored by assigning a black color to it.
		if(positionInsideVolume.x>= 0.0 && positionInsideVolume.x<= 1.0 && positionInsideVolume.y>= 0.0 && positionInsideVolume.y<= 1.0 && positionInsideVolume.z>= 0.0 && positionInsideVolume.z<= 1.0){
			float layer= floor(positionInsideVolume.y*propagationVolumeHeightResolution);
			position.x+= lightPropagationVolumeWidth*layer;
		}
		else
			color= vec3(0.0);

		gl_Position= gl_ModelViewProjectionMatrix*vec4(position, 1.0);
		lightColor= color;
		lightDirection= normal;
	}
	"

	includes{
		file= resources/shaders/engine/atmosphere/skyLighting/skyLightingVolumeRead.sh
	}
	
	semantics{
		depthTexture= GBUFFER_DEPTH_TEXTURE
		colorTexture= GBUFFER_COLOR_TEXTURE
		positionTexture= GBUFFER_POSITION_TEXTURE
		normalTexture= GBUFFER_NORMAL_TEXTURE
		propagationVolumeWorldSpaceTexelSize= LIGHT_PROPAGATION_VOLUME_WORLDSPACE_TEXEL_SIZE
		propagationVolumeLightsWeight= LIGHT_PROPAGATION_VOLUME_LIGHTS_WEIGHT
		lightPropagationVolumePosition= LIGHT_PROPAGATION_VOLUME_POSITION
		lightPropagationVolumeWidth= LIGHT_PROPAGATION_VOLUME_WIDTH
		lightPropagationVolumeHeight= LIGHT_PROPAGATION_VOLUME_HEIGHT
		lightPropagationVolumeLength= LIGHT_PROPAGATION_VOLUME_LENGTH
		propagationVolumeHeightResolution= LIGHT_PROPAGATION_VOLUME_HEIGHT_RESOLUTION
	}
}