shader{

	code= "
	//Spherical Harmonics coefficients.
	//Unlike Crytek's documentation, these coefficients have no physical or mathematical accuracy
	//and thus were tunned to get the most reallistic results. In practice this makes sense because
	//we're representing a radiance function with only 4 coefficents which is always very inaccurate
	//whether we use accurate or innacurate coefficients, so we can just tune them as we see fit.
	#define C1		0.4
	#define C2		0.6
	
	struct SH{
		vec3 L00;
		vec3 L1m1;
		vec3 L10;
		vec3 L11;
	};
	
	SH createLightLobeSH(vec3 lightColor, vec3 lightDirection){
		SH result;
		result.L00= lightColor*C1;
		result.L1m1= lightColor*lightDirection.y*C2;
		result.L10= lightColor*lightDirection.z*C2;
		result.L11= lightColor*lightDirection.x*C2;
		return result;
	}

	SH addSH(SH sh1, SH sh2){
		SH result;
		result.L00= sh1.L00+ sh2.L00;
		result.L1m1= sh1.L1m1+ sh2.L1m1;
		result.L10= sh1.L10+ sh2.L10;
		result.L11= sh1.L11+ sh2.L11;
		
		return result;
	}
	
	vec3 sampleColorFromSH(SH sh, vec3 direction){
		return max(sh.L00*C1+ (sh.L1m1*direction.y+ sh.L10*direction.z+ sh.L11*direction.x)*C2, 0.0);
	}
	"
}