shader
{
	code= "
	#define				NUM_SAMPLES 15
	#define				BLUR_RADIUS 0.02

	uniform sampler2D	unblurredCircleOfConfusionTexture;
	uniform sampler2D	circleOfConfusionTexture;
	uniform vec2		blurDirectionAndScale;

	void main()
	{
		//Get the pixel information.
		float depth = getPixelDepth(gl_TexCoord[0].st);
		float circleOfConfusion = texture(unblurredCircleOfConfusionTexture, gl_TexCoord[0].st);

		//Setup some data for the blur.
		vec2 texCoord = gl_TexCoord[0].st - blurDirectionAndScale * BLUR_RADIUS * 0.5;
		vec2 texCoordStep = blurDirectionAndScale * BLUR_RADIUS / (NUM_SAMPLES - 1);
		float result = 0.0;
		float totalWeight = 1.0;

		//Perform the blur.
		for(unsigned int i = 0; i < NUM_SAMPLES; i++, texCoord += texCoordStep)
		{
			//Get the current circle of confusion value for this sample.
			float sampleCircleOfConfusion = texture(circleOfConfusionTexture, texCoord).x;
			
			//Get the sample's depth and original circle of confusion.
			float sampleDepth = getPixelDepth(texCoord);
			float originalSampleCircleOfConfusion = texture(unblurredCircleOfConfusionTexture, texCoord);

			float weight = 0.0;

			//If the current pixel is in focus...
			if(circleOfConfusion < 0.01)
			{
				//Sample only from foreground pixels.
				if(sampleDepth <= depth)
				{
					weight = 1;
				}
			}
			else
			{
				//If not in focus then sample from pixels with greater depth.
				if(sampleDepth >= depth)
				{
					weight = 1;
				}

				//Sample from foreground pixels that are also out of focus.
				if(sampleDepth <= depth && originalSampleCircleOfConfusion > 0.001)
				{
					weight = 1;
				}
			}

			//Accumulate the weighted sample.
			result += sampleCircleOfConfusion * weight;
			totalWeight += weight;
		}

		//Store the normalized resutlt.
		gl_FragColor = result / totalWeight;
	}
	"

	includes
	{
		file = resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
	}
}