shader
{
	code= "
	#define				NUM_SAMPLES 31

	uniform sampler2D	sourceColorTexture;
	uniform sampler2D	circleOfConfusionTexture;
	uniform vec2		blurDirectionAndScale;
	uniform float		gaussianCoefficients[NUM_SAMPLES];

	void main()
	{
		float circleOfConfusion = texture2D(circleOfConfusionTexture, gl_TexCoord[0].st).x;
		float depth = getPixelDepth(gl_TexCoord[0].st);

		vec2 texCoord = gl_TexCoord[0].st - blurDirectionAndScale * circleOfConfusion * 0.5;
		vec2 texCoordStep = blurDirectionAndScale * circleOfConfusion / (NUM_SAMPLES - 1);
		vec3 finalColor = 0.0;
		float totalWeight = 0.0;

		for(unsigned int i = 0; i < NUM_SAMPLES; i++, texCoord += texCoordStep)
		{
			vec3 sampleColor = texture2D(sourceColorTexture, texCoord).rgb;
			float sampleCircleOfConfusion = texture2D(circleOfConfusionTexture, texCoord).x;
			float sampleDepth = getPixelDepth(texCoord);

			float gaussianWeight = gaussianCoefficients[i];
			
			//Ignore samples that belong to a foreground objects that are in focus. To do this
			//we apply a weighting function to check if the sample is in focus and another one
			//to check if the sample is closer than the center pixel. Then we subtract the two
			//combined weights from the current weight of the sample.
			float inFocusWeight= (1.0 - min(sampleCircleOfConfusion / 0.01, 1.0));
			float depthWeight = clamp((depth - sampleDepth) / 0.1, 0.0, 1.0);
			float weight = gaussianWeight * max(1.0 - inFocusWeight * depthWeight, 0.0);

			//Accumulate the sample.
			finalColor += sampleColor * weight;
			totalWeight += weight;
		}

		gl_FragColor = vec4(finalColor / totalWeight, 1.0);
	}
	"

	includes
	{
		file = resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
	}
}