shader{

	code= "
	#define AIR_REFRACTION_INDEX	1.0
	#define GLASS_REFRACTION_INDEX	1.52

	uniform vec3	cameraPosition;

	varying vec3	position;
	varying vec3	normal;
			
	void main()
	{

		const float CONE_ANGLE = 0.001f;

		vec3 viewVector = normalize(cameraPosition - position);
		vec3 reflectionDirection = normalize(reflect(-viewVector, normal));
		vec4 reflectionCone = traceReflectionCone(position, normal, reflectionDirection, CONE_ANGLE);

		vec3 skyLighting = sampleSkyMap(reflectionDirection, CONE_ANGLE) * reflectionCone.a;
		vec3 reflection = reflectionCone.rgb + skyLighting;

		//Invert the normal if this is a back face.
		if(dot(viewVector, normal)< 0.0)
			normal= -normal;
				
		//Fresnel equation for the calculation of the reflection coefficient of light when moving from air to glass.
		//This equation is available on Wikipedia here: http://en.wikipedia.org/wiki/Fresnel_equations.
		float incidentAngleCosine= dot(viewVector, normalize(normal));
		float incidentAngleSine= 1.0- incidentAngleCosine*incidentAngleCosine;
		float refractionAngleSine= incidentAngleSine*AIR_REFRACTION_INDEX/GLASS_REFRACTION_INDEX;
		float refractionAngleCosine= sqrt(1.0- refractionAngleSine*refractionAngleSine);
		float aux1= AIR_REFRACTION_INDEX*incidentAngleCosine;
		float aux2= GLASS_REFRACTION_INDEX*refractionAngleCosine;
		float aux3= (aux1- aux2)/(aux1+ aux2);
		float fresnel= aux3*aux3;

		//Wikipedia also features this equation to better approximate the
		//behaviour of light when traveling through common glass.
		fresnel= 2.0*fresnel/(1.0 + fresnel);


		gl_FragColor= vec4(reflection * fresnel, fresnel);
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/voxelConeTracing.sh
		file = resources/shaders/engine/atmosphere/skyLighting/skyMapRead.sh
	}

	semantics
	{
		cameraPosition = CAMERA_POSITION
	}
}