shader{

	code= "
	#define AIR_REFRACTION_INDEX	1.0
	#define GLASS_REFRACTION_INDEX	1.2

	uniform sampler2D	reflectionsTexture;
	uniform vec3		cameraPosition;

	varying vec3		normal;
	varying vec3		position;
			
	void main(){

		vec2 texCoord= gl_TexCoord[1].xy/gl_TexCoord[1].w;
		vec3 reflection= texture2D(reflectionsTexture, texCoord).rgb;

		vec3 viewVector= normalize(cameraPosition- position);
				
		//Fresnel equation for the calculation of the reflection coefficient of light when moving from air to glass.
		//This equation is available on Wikipedia here: http://en.wikipedia.org/wiki/Fresnel_equations.
		float incidentAngleCosine= dot(viewVector, normalize(normal));
		float incidentAngleSine= 1.0- incidentAngleCosine*incidentAngleCosine;
		float refractionAngleSine= incidentAngleSine*AIR_REFRACTION_INDEX/GLASS_REFRACTION_INDEX;
		float refractionAngleCosine= sqrt(1.0- refractionAngleSine*refractionAngleSine);
		float aux1= AIR_REFRACTION_INDEX*incidentAngleCosine;
		float aux2= GLASS_REFRACTION_INDEX*refractionAngleCosine;
		float aux3= (aux1- aux2)/(aux1+ aux2);
		float fresnel= aux3*aux3;

		//Wikipedia also features this equation to better approximate the
		//behaviour of light when traveling through common glass.
		fresnel= 2.0*fresnel/(1.0 + fresnel);

		gl_FragColor= vec4(0.0);
	}
	"

	semantics{
		reflectionsTexture= RAYTRACING_REFLECTIONS_TEXTURE
		cameraPosition= CAMERA_POSITION
	}
}