shader{

	code= "
	uniform sampler2D	reflectionsTexture;
	uniform sampler2D	colorTexture;
	uniform sampler2D	normalTexture;
	uniform vec3		cameraPosition;

	varying vec3		normal;
	varying vec3		position;
			
	void main(){

		vec2 texCoord= gl_TexCoord[1].xy/gl_TexCoord[1].w;
		vec3 reflection= texture2D(reflectionsTexture, texCoord).rgb;
		float mask=  texture2D(colorTexture, gl_TexCoord[0].st).a;

		vec3 viewVector= normalize(cameraPosition- position);
		float fresnel= pow(1.0- max(dot(normal, viewVector), 0.0), 4.0);
		
		//Avoid excessive reflection.
		fresnel= min(fresnel, 0.2);
		
		gl_FragColor= vec4(reflection*fresnel*mask, fresnel);
	}
	"

	semantics{
		reflectionsTexture= RAYTRACING_REFLECTIONS_TEXTURE
		colorTexture= MATERIAL_COLOR_TEXTURE
		normalTexture= GBUFFER_NORMAL_TEXTURE
		cameraPosition= CAMERA_POSITION
	}
}